#ifndef MATRIX_H_
#define MATRIX_H_

#include <stdexcept>
#include <iostream>

using namespace std;

class Matrix
  {
    public:

        /**
         * Construct a Matrix with the specified dimensions.
         *
         * @throws invalid_argument if either dimension is less than or equal
         *         to zero.
         */
        Matrix(int nRows, int nCols) throw(invalid_argument);

        /**
         * Copy constructor.
         */
        Matrix(const Matrix& m);

        /**
         * Destructor.
         */
        ~Matrix();

        /**
         * Assignment operator.
         */
        Matrix& operator=(const Matrix& rhs);

        /**
         * Matrix index operator.  Returns reference for use on left side of
         * assignment.
         *
         * @throws invalid_argument if specified indexes are not within Matrix
         *         dimensions.
         */
        double& operator()(int x, int y) throw(invalid_argument);

        /**
         * Matrix index operator.  Returns value for use in expressions.
         *
         * @throws invalid_argument if specified indexes are not within Matrix
         *         dimensions.
         */
        double operator()(int x, int y) const throw(invalid_argument);

        /**
         * Matrix addition.
         *
         * @throws invalid_argument if specified Matrices are not compatible
         *         for addition.
         */
        friend Matrix operator+(const Matrix& m1, const Matrix& m2)
            throw(invalid_argument);

        /**
         * Matrix subtraction.
         *
         * @throws invalid_argument if specified Matrices are not compatible
         *         for subtraction.
         */
        friend Matrix operator-(const Matrix& m1, const Matrix& m2)
            throw(invalid_argument);

        /**
         * Matrix multiplication.
         *
         * @throws invalid_argument if specified Matrices are not compatible
         *         for multiplication.
         */
        friend Matrix operator*(const Matrix& m1, const Matrix& m2)
            throw(invalid_argument);

        /**
         * Scalar multiplication.  Scalar on left side of operator.
         */
        friend Matrix operator*(double x, const Matrix& m);

        /**
         * Scalar multiplication.  Scalar on right side of operator.
         */
        friend Matrix operator*(const Matrix& m, double x);

        /**
         * Output operator.
         */
        friend ostream &operator<<(ostream &out, const Matrix &m);

    private:

        int nRows, nCols;   // dimensions of the matrix
        double **p;         // matrix implemented as pointer to a pointer

        void allocMatrix();
        static bool checkArgs(Matrix a, Matrix b);
  };


inline double& Matrix::operator()(int i, int j) throw(invalid_argument)
  {
    if (i < 0 || i >= nRows || j < 0 || j > nCols)
        throw invalid_argument("Invalid subscript");

    return p[i][j];
  }

inline double Matrix::operator()(int i, int j) const throw(invalid_argument)
  {
    if (i < 0 || i >= nRows || j < 0 || j > nCols)
        throw invalid_argument("Invalid subscript");

    return p[i][j];
  }

#endif
